#ifndef Wykresh
#define Wykresh

#include <vector>
#include <gl\gl.h>
#include <gl\glu.h>   // Plik nagwkowy biblioteki GLu32
#include "Solution.h"

#define MAX_CHARS 256
#define GRIDW 100
#define GRIDH 100
#define QUADW (GRIDW-1)
#define QUADH (GRIDH-1)
#define VERTEXNUM (GRIDW*GRIDH)
#define QUADNUM (QUADW*QUADH)

typedef struct Vertex
{
  GLfloat x,y,z;
  GLfloat r,g,b;
} Vertex2;

typedef std::vector<float> vVecData1D;
typedef std::vector<vVecData1D> vVecData2D;

class COpenGLView
{
 private:
   HDC hDC;
   HGLRC hRC;
   HFONT hOldFont;
   int fontListID;

   bool bDemoLastState;
   //! Zmienna wykorzystywana w demonstaracji
   float dt;
   //! Struktura wykorzystywana w demonstaracji
   float p[GRIDW][GRIDH];
   //! Struktura wykorzystywana w demonstaracji
   float vx[GRIDW][GRIDH],vy[GRIDW][GRIDH];
   //! Struktura wykorzystywana w demonstaracji
   float ax[GRIDW][GRIDH],ay[GRIDW][GRIDH];
   //! Metoda wykorzystywana w demonstracji - wyliczajaca powierzchnie
   void calcSurface();
   //! Metoda wykorzystywana w demonstracji - dostosowujaca wspolrzedna z i wyznaczajaca kolor punktow
   void adjustGrid();
   //! Metoda wykorzystywana w demonstracji - inicjalizujaca wyswietlanie powierzchnie
   void initSurface();
   //! Metoda inicjalizujaca tabele okreslajace sposob wyswietlania punktow
   void initVertices();
   //! Zmienna wskazujca na koniecznosc inicjalizacji tablic wykorzystywanych w wyswietlaniu wykresow
   bool bInitial;

   //! Warosci min i max wszystkich 3 osi
   float MaxX,MinX;
   float MaxY,MinY;
   float MaxZ,MinZ;
   //! Warosci min i max wszystkich 3 osi dostosowane do sposobu wyswietlania wykresu
   float MaxXA,MinXA;
   float MaxYA,MinYA;
   float MaxZA,MinZA;

   //! Dane wykresu - tablica punktow oraz kolorow punktow
   Vertex vertex[VERTEXNUM];
   //! Tablica okreslajaca sposob wyswietlania punktow wykresu
   GLuint quad[4*QUADNUM];

   //! Dane prezentowanego rozwiazania lokalnego - wyniku optymalizacji
   std::vector<float> vLocalSolution;
   //! Dane prezentowanego rozwiazania globalnego - wyniku optymalizacji
   std::vector<float> vGASolution;

   //! Metoda tworzy czcionke
   void CreateFonts(HDC hDC);
   //! Metoda usuwa czcionke
   void DeleteFonts(HDC hDC);
   //! Metoda wypisuje tekst na ekranie
   void WriteText(float x,float y,float z,char * text);
   //! Przygotowuje format piksela do rysowania w zadanym oknie
   void SetDCPixelFormat(HDC hDC);
   //! Ustawia parametry OpenGL dla okreslonego rozmiaru okna
   //! Mozna tej funkcji uzywac wielokrotnie do przeskalowywania (onResize)
   void InitOpenGL(RECT rect);
   //! Czyszczenie ekranu
   void Clear(void);

   double mod(double value, double modulo);
   //! Metoda rysuje pojedynczy punkt rozwiazania lokalnego
   void DrawLocalSolutionPoint(int j, float fUnitX, float fUnitY, float fRangeZ, bool bDrawSphere);
   //! Metoda rysuje pojedynczy punkt rozwiazania globalnego
   void DrawGASolutionPoint(int j, float fUnitX, float fUnitY, float fRangeZ, bool bDrawSphere);
   //! Metoda rysuje pojedynczy punkt rozwiazania lokalnego  na wykresie dwuwymiarowym
   void DrawLocalSolutionPoint2D(int j, float fUnitX, float fUnitY, float fRangeZ, bool bDrawSphere);
   //! Metoda rysuje pojedynczy punkt rozwiazania globalnego na wykresie dwuwymiarowym
   void DrawGASolutionPoint2D(int j, float fUnitX, float fUnitY, float fRangeZ, bool bDrawSphere);
   //! Obiekt wykorzystywany do narysowania sfery
   GLUquadricObj *quadratic;

 public:
  //! Konstruktor
  COpenGLView(HDC hDC);
  //! Destruktor
  ~COpenGLView();

  //! Przygotowanie danych do wyswietlenia na wykresie
  void SetVertexData(float MinX, float MaxX, float MinY, float MaxY, float MinZ, float MaxZ,
                      vVecData1D vVertexZ, std::vector <float> vLocalSol, std::vector <float> vGASol);
  //! Rysowanie wykresu
  void Render(RECT rect, bool bDemo, float Zoom, float kx, float ky, float PosX, float PosY,
              bool Wireframe, bool Polygon, bool Shading, bool bSolution,
              bool Axes, bool DescAxes, bool Frames, bool bSpiral);
};

#endif

