//---------------------------------------------------------------------------
#pragma hdrstop
#include "GlobalSearch.h"
#include "OptimizationThread.h"
#include "MainForm.h"
#include <math.h>
#include <stdlib.h>

#define max(a, b)  (((a) > (b)) ? (a) : (b))

//! Konstruktor klasy
CGlobalSearch::CGlobalSearch(void)
{
}

//! Destruktor klasy
CGlobalSearch::~CGlobalSearch(void)
{
}

CSolution CGlobalSearch::Initial(CAlgParameters &AlgParam, int iPopulationSize)
{
   int iRange;
   float fValue;
   double vals[MAXVAR];
   std::vector <float> InitialVector;
   CSolution Solution;

   randomize();
   Solution.Clear();

   // parsowanie funkcji
   int ret = AlgParam.FuncParser.Parse(AlgParam.Function, AlgParam.FuncVars);
   if(ret >= 0) return Solution;

   // wylosuj wartosci zmiennych
   for (int j = 0; j < iPopulationSize; j++)
   {
      InitialVector.clear();
      for (int i = 0; i < AlgParam.Variables.GetNumberOfVar(); i++)
      {
         float fMaxValue = AlgParam.Variables.GetVarMax(i);
         float fMinValue = AlgParam.Variables.GetVarMin(i);
         float fRange = fMaxValue - fMinValue;
         int ii = 0;
         float fRand = fRange;
         for (ii; fRand < 100; ii++) fRand *= 10;
         fValue = (float)(rand() % (int)fRand);
         for (ii; ii > 0; ii--) fValue /= 10;

         vals[i] = fValue + fMinValue;
         if (vals[i] > fMaxValue)
            vals[i] = fMaxValue;
         InitialVector.push_back(vals[i]);
      }
      Solution.Add(InitialVector, AlgParam.FuncParser.Eval(vals));     // zapamietanie rozwiazania
   }
   return Solution;
}
//******************************************************************************

CSolution CGlobalSearch::TournamentSelection(CSolution Population, int iTournamentSize, bool bOptimMax)
{
   int iSize = Population.GetNumberOfSol();
   float fBest, fTemp;
   CSolution NewPopulation;

   NewPopulation.Clear();
   for (int j = 0; j < iSize; j++)
   {
      fBest = rand() % iSize;   //losujemy pierwszego
      for (int i = 0; i <= iTournamentSize; i++)   //losujemy kolejnych i sprawdzamy czy sa lepsi
      {
         fTemp = rand() % iSize;
         if (!bOptimMax && (Population.GetFunc(fBest) > Population.GetFunc(fTemp)))
            fBest = fTemp;
         if (bOptimMax && (Population.GetFunc(fBest) < Population.GetFunc(fTemp)))
            fBest = fTemp;
      }
      NewPopulation.AddSolution(Population.GetSolution(fBest));
   }
   return NewPopulation;
}
//---------------------------------------------------------------------------

CSolution CGlobalSearch::RouletteSelection(CSolution Population, bool bOptimMax)
{
   int iSize = Population.GetNumberOfSol();
   float fWorst, fRatioRoulette;
   float fFitness;
   std::vector<float> vProbability;
   std::vector<float> vDystryb;
   CSolution NewPopulation;

   fFitness = 0;
   fWorst = Population.GetFunc(0);
   NewPopulation.Clear();
   vProbability.clear();
   vDystryb.clear();

   if (!bOptimMax)
   {
      for(int i = 0; i < iSize; i++)  // szukamy najgorszego - o najwiekszej fc
         if (fWorst < Population.GetFunc(i)) fWorst = Population.GetFunc(i);
      for(int i = 0; i < iSize; i++)  // liczymy dopasowanie populacji
         fFitness += fWorst - Population.GetFunc(i); //poniewaz minimalizujemy funkcje celu
      for(int i = 0; i < iSize; i++) // liczymy prawdopodobienstwo
         if (fFitness == 0.0) vProbability.push_back(0.1);
         else vProbability.push_back((fWorst - Population.GetFunc(i)) / fFitness);
      for(int i = 0; i < iSize; i++) // liczymy dystrybuanty
         if (i == 0) vDystryb.push_back(vProbability[0]);
         else vDystryb.push_back(vProbability[i] + vDystryb[vDystryb.size() - 1]);
   }
   else
   {
      for(int i = 0; i < iSize; i++)  // szukamy najgorszego - o najmniejszej fc
         if (fWorst > Population.GetFunc(i)) fWorst = Population.GetFunc(i);
      for(int i = 0; i < iSize; i++)  // liczymy dopasowanie populacji
         fFitness += Population.GetFunc(i) - fWorst; //poniewaz minimalizujemy funkcje celu
      for(int i = 0; i < iSize; i++) // liczymy prawdopodobienstwo
         if (fFitness == 0.0) vProbability.push_back(0.1);
         else vProbability.push_back((Population.GetFunc(i) - fWorst) / fFitness);
      for(int i = 0; i < iSize; i++) // liczymy dystrybuanty
         if (i == 0) vDystryb.push_back(vProbability[0]);
         else vDystryb.push_back(vProbability[i] + vDystryb[vDystryb.size() - 1]);
   }

   for(int i = 0; i < iSize; i++) // selekcjonujemy
   {
      fRatioRoulette = (float)random(100)/100;
      int j;
      for (j = 0; j < iSize; j++)
         if (fRatioRoulette <= vDystryb[j]) break;
      NewPopulation.AddSolution(Population.GetSolution(j));
   }
   return NewPopulation;
}
//---------------------------------------------------------------------------

CSolution CGlobalSearch::Crossover1Point(CSolution Population, CAlgParameters &AlgParam)
{
   int iSize = Population.GetNumberOfSol();
   int iVarSize = Population.GetNumberOfVar();
   int iPoint, iX;
   int iOne;
   int iFirst = 0;
   double vals[MAXVAR], vals2[MAXVAR];
   std::vector <float> NewChild1;
   std::vector <float> NewChild2;
   CSolution NewPopulation;
   NewPopulation.Clear();

  if (iVarSize > 0)
   for(int iTwo = 0; iTwo < iSize; iTwo++)
   {
      NewChild1.clear();
      NewChild2.clear();
      iX = rand() % 1000 /1000;
      if ((iX < AlgParam.fCOProbability) && (iTwo < (iSize - 1)))
      {
         ++iFirst;
         if (iFirst % 2 != 0)
         {
            iOne = iTwo;
         }
         else
         {
            if (iSize > 1)
            {
               if (iSize == 2)
                  iPoint = 1;
               else
                  iPoint = rand() % (iVarSize - 1) + 1;
            }
            for (int j = 0; j < Population.GetNumberOfVar(); j++)
               if (j < iPoint)
               {
                  NewChild1.push_back(Population.GetSolutionVar(iOne, j));
                  NewChild2.push_back(Population.GetSolutionVar(iTwo, j));
                  vals[j] = Population.GetSolutionVar(iOne, j);
                  vals2[j] = Population.GetSolutionVar(iTwo, j);
               }
               else
               {
                  NewChild1.push_back(Population.GetSolutionVar(iTwo, j));
                  NewChild2.push_back(Population.GetSolutionVar(iOne, j));
                  vals[j] = Population.GetSolutionVar(iTwo, j);
                  vals2[j] = Population.GetSolutionVar(iOne, j);
               }
            NewPopulation.Add(NewChild1, AlgParam.FuncParser.Eval(vals));
            NewPopulation.Add(NewChild2, AlgParam.FuncParser.Eval(vals2));
         }
      }
      else
      {
         NewPopulation.AddSolution(Population.GetSolution(iTwo));
      }
   }
   return NewPopulation;
}
//---------------------------------------------------------------------------

CSolution CGlobalSearch::Crossover2Point(CSolution Population, CAlgParameters &AlgParam)
{
   int iSize = Population.GetNumberOfSol();
   int iVarSize = Population.GetNumberOfVar();
   int iPointFirst, iPointSecond, iX;
   int iOne;
   int iFirst = 0;
   double vals[MAXVAR], vals2[MAXVAR];
   std::vector <float> NewChild1;
   std::vector <float> NewChild2;
   CSolution NewPopulation;
   NewPopulation.Clear();

  if (iVarSize > 0)
   for(int iTwo = 0; iTwo < iSize; iTwo++)
   {
      NewChild1.clear();
      NewChild2.clear();
      iX = rand() % 100;
      if ((iX < AlgParam.fCOProbability) && (iTwo < (iSize - 1)))
      {
         ++iFirst;
         if (iFirst % 2 != 0)
         {
            iOne = iTwo;
         }
         else
         {
            if (iSize > 1)
            {
               if (iSize == 2)
               {
                  iPointFirst = 1;
                  iPointSecond = 2;
               }
               else
               {
                  iPointFirst = rand() % (iVarSize - 1) + 1;
                  iPointSecond = rand() % (iSize - iPointFirst) + iPointFirst;
               }
            }
            for (int j = 0; j < Population.GetNumberOfVar(); j++)
               if ((j < iPointFirst) || (j >= iPointSecond))
               {
                  NewChild1.push_back(Population.GetSolutionVar(iOne, j));
                  NewChild2.push_back(Population.GetSolutionVar(iTwo, j));
                  vals[j] = Population.GetSolutionVar(iOne, j);
                  vals2[j] = Population.GetSolutionVar(iTwo, j);
               }
               else
               {
                  NewChild1.push_back(Population.GetSolutionVar(iTwo, j));
                  NewChild2.push_back(Population.GetSolutionVar(iOne, j));
                  vals[j] = Population.GetSolutionVar(iTwo, j);
                  vals2[j] = Population.GetSolutionVar(iOne, j);
               }
            NewPopulation.Add(NewChild1, AlgParam.FuncParser.Eval(vals));
            NewPopulation.Add(NewChild2, AlgParam.FuncParser.Eval(vals2));
         }
      }
      else
      {
         NewPopulation.AddSolution(Population.GetSolution(iTwo));
      }
   }
   return NewPopulation;
}
//---------------------------------------------------------------------------

CSolution CGlobalSearch::CrossoverArythmetic(CSolution Population, CAlgParameters &AlgParam)
{
   int iSize = Population.GetNumberOfSol();
   int iVarSize = Population.GetNumberOfVar();
   int iPoint, iX;
   int iOne;
   int iFirst = 0;
   double vals[MAXVAR], vals2[MAXVAR];
   std::vector <float> NewChild1;
   std::vector <float> NewChild2;
   CSolution NewPopulation;
   NewPopulation.Clear();
   float a = AlgParam.fCOA;

  if (iVarSize > 0)
   for(int iTwo = 0; iTwo < iSize; iTwo++)
   {
      NewChild1.clear();
      NewChild2.clear();
      iX = rand() % 1000 /1000;
      if ((iX < AlgParam.fCOProbability) && (iTwo < (iSize - 1)))
      {
         ++iFirst;
         if (iFirst % 2 != 0)
         {
            iOne = iTwo;
         }
         else
         {
            for (int j = 0; j < Population.GetNumberOfVar(); j++)
            {
               float x1 = Population.GetSolutionVar(iOne, j);
               float x2 = Population.GetSolutionVar(iTwo, j);
               float x1prim = a * x1 + (1 - a) * x2;
               float x2prim = a * x2 + (1 - a) * x1;

               NewChild1.push_back(x1prim);
               NewChild2.push_back(x2prim);
               vals[j] = x1prim;
               vals2[j] = x2prim;
            }
            NewPopulation.Add(NewChild1, AlgParam.FuncParser.Eval(vals));
            NewPopulation.Add(NewChild2, AlgParam.FuncParser.Eval(vals2));
         }
      }
      else
      {
         NewPopulation.AddSolution(Population.GetSolution(iTwo));
      }
   }
   return NewPopulation;
}

//---------------------------------------------------------------------------

CSolution CGlobalSearch::Mutation(CSolution Population, CAlgParameters &AlgParam)
{
   int iX, iSize = Population.GetNumberOfSol();
   int iMutPos, iVarSize = Population.GetNumberOfVar();
   float fValue, fRange, fNewPart;
   float fMinValue, fMaxValue, fLastValue;
   double vals[MAXVAR];
   std::vector <float> NewVector;
   CSolution NewPopulation;
   NewPopulation.Clear();

  if (iVarSize > 0)
   for(int i = 0; i < iSize; i++)
   {
      iX = rand() % 100;
      NewVector.clear();
      if (iX < (int)AlgParam.fMProbability)
      {
         iMutPos = rand() % iVarSize;
         for (int j = 0; j < iVarSize; j++)
         {
            if (j != iMutPos)
            {
               vals[j] = Population.GetSolutionVar(i, j);
               NewVector.push_back(vals[j]);
            }
            else
            {
               fLastValue = Population.GetSolutionVar(i, iMutPos);
               fMinValue = AlgParam.Variables.GetVarMin(iMutPos);
               fMaxValue = AlgParam.Variables.GetVarMax(iMutPos);
               fRange = (fMaxValue - fMinValue) * AlgParam.fRangeMutA;
               int ii = 0;
               float fRand = fRange;
               fRand *= 2; // dla symetrycznosci przedzialu wartosci ujemne i dodatnie
               for (ii; fRand < 100; ii++) fRand *= 10;
               fNewPart = (float)(rand() % (int)fRand);
               for (ii; ii > 0; ii--) fNewPart /= 10;

               if (fNewPart < fRange)
                  fValue = fLastValue - fNewPart;
               else
                  fValue = fLastValue + fNewPart / 2;
               if (fValue > fMaxValue)
                  fValue = fMaxValue - (fValue - fMaxValue);
               if (fValue < fMinValue)
                  fValue = fMinValue + (fMinValue - fValue);
               NewVector.push_back(fValue);
               vals[j] = fValue;
            }
         }
         NewPopulation.Add(NewVector, AlgParam.FuncParser.Eval(vals));
      }
      else
      {
         NewPopulation.AddSolution(Population.GetSolution(i));
      }
  }
  return NewPopulation;
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

TSol CGlobalSearch::Check(TSol BestSolution, TSol BestPopulSol, bool bOptimMax)
{
   if (bOptimMax)
   {
      if (BestSolution.func < BestPopulSol.func)
         BestSolution = BestPopulSol;
   }
   else
   {
      if (BestSolution.func > BestPopulSol.func)
         BestSolution = BestPopulSol;
   }
   return BestSolution;
}
//---------------------------------------------------------------------------

CSolution CGlobalSearch::GeneticAlgorithm(CAlgParameters &AlgParam)
{
   TSol BestSolution;

   Solution.Clear();
   Population.Clear();

   // Inicjalizacja
   Population = Initial(AlgParam, AlgParam.iPopSize);
   BestSolution = Population.GetBestSolution(AlgParam.OptimMax);
   Sleep(AlgParam.iGeneticSleep);
   // Populacja
   if (Population.GetNumberOfVar() > 0)
   for (int i = 0; i < AlgParam.iAlgIterat; i++)
   {
      if (FormMain->bStop) break;
      switch (AlgParam.iSelectionType)
      {
         case 0 : Population = TournamentSelection(Population, AlgParam.iTournamentSize, AlgParam.OptimMax);break;
         case 1 : Population = RouletteSelection(Population, AlgParam.OptimMax);break;
      }
      /////////////////////////////////////////////////////////////
      if (FormMain->bStop) break;
      if (Population.GetNumberOfVar() > 1)
         switch (AlgParam.iCrossOverType)
         {
            case 0 : Population = Crossover1Point(Population, AlgParam); break;
            case 1 : Population = Crossover2Point(Population, AlgParam); break;
            case 2 : Population = CrossoverArythmetic(Population, AlgParam); break;
         }
      /////////////////////////////////////////////////////////////
      if (FormMain->bStop) break;
      Population = Mutation(Population, AlgParam);
      BestSolution = Check(BestSolution, Population.GetBestSolution(AlgParam.OptimMax), AlgParam.OptimMax);
      /////////////////////////////////////////////////////////////
      Solution.AddSolution(BestSolution);
      if (FormMain->bStop) break;
      Sleep(AlgParam.iGeneticSleep);
   }
   return Solution;
}
//---------------------------------------------------------------------------

CSolution CGlobalSearch::GetSolution(void)
{
   return Solution;
}

CSolution CGlobalSearch::GetPopulation(void)
{
   return Population;
}

#pragma package(smart_init)
