#include <math.h>
#include <stdlib.h>
#include "LocalSearch.h"
#include "MainForm.h"

#define max(a, b)  (((a) > (b)) ? (a) : (b))

//! Konstruktor klasy
CLocalSearch::CLocalSearch(void)
{
}

//! Destruktor klasy
CLocalSearch::~CLocalSearch(void)
{
}

CSolution CLocalSearch::Initial(CAlgParameters &AlgParam)
{
   int iRange;
   float fValue;
   double vals[MAXVAR];
   std::vector <float> InitialVector;

   randomize();
   Solution.Clear();
   InitialVector.clear();

   // parsowanie funkcji
   int ret = AlgParam.FuncParser.Parse(AlgParam.Function, AlgParam.FuncVars);
   if(ret >= 0) return Solution;

   // wylosuj wartosci zmiennych
   for (int i = 0; i < AlgParam.Variables.GetNumberOfVar(); i++)
   {
      iRange = (int)((AlgParam.Variables.GetVarMax(i) - AlgParam.Variables.GetVarMin(i))*100);
      if (iRange != 0)
         fValue = (float)(rand() % iRange) / 100;
      else
         fValue = 0;
      vals[i] = fValue + AlgParam.Variables.GetVarMin(i);
      InitialVector.push_back(vals[i]);
   }

   Solution.Add(InitialVector, AlgParam.FuncParser.Eval(vals));     // zapamietanie rozwiazania
   return Solution;
}

//******************************************************************************
void CLocalSearch::Neighbourhood(CAlgParameters &AlgParam, CSolution &Solution, int sol, double vals[])
{
   int iRange;
   float fMinValue, fMaxValue, fLastValue, fNewValue, fRange;

   int iNrOfVar = AlgParam.Variables.GetNumberOfVar();
   // generowanie calego ssiedztwa
   for (int i = 0; i < iNrOfVar; i++)
   {
      fMinValue = AlgParam.Variables.GetVarMin(i);
      fMaxValue = AlgParam.Variables.GetVarMax(i);
      fLastValue = Solution.GetLastSolVar(i);
      fRange = (fMaxValue - fMinValue) * AlgParam.fRangeA;

      int pos = pow(2, i);
      // sprawdzamy czy na okreslonej pozycji jest 1
      if (sol & pos)
      {
         int ii = 0;
         float fNewPart, fRand = fRange;
         for (ii; fRand < 100; ii++) fRand *= 10;
         fNewPart = (float)(rand() % (int)fRand);
         for (ii; ii > 0; ii--) fNewPart /= 10;

         fNewValue = fLastValue + fNewPart;
         if (fNewValue > fMaxValue)
            fNewValue = fLastValue;
      }
      else
      {
         int ii = 0;
         float fNewPart, fRand = fRange;
         for (ii; fRand < 100; ii++) fRand *= 10;
         fNewPart = (float)(rand() % (int)fRand);
         for (ii; ii > 0; ii--) fNewPart /= 10;

         fNewValue = fLastValue - fNewPart;
         if (fNewValue < fMinValue)
            fNewValue = fLastValue;
      }
      // zapisujaemy wartosc zmiennej
      vals[i] = fNewValue;
   }
}

//******************************************************************************

void CLocalSearch::NeighbourhoodSA(CAlgParameters &AlgParam, CSolution &Solution, int sol,
                                 double vals[], double newvals[], bool bTempOk)
{
   int iRange;
   float fMinValue, fMaxValue, fLastValue, fNewValue, fRange;

   int iNrOfVar = AlgParam.Variables.GetNumberOfVar();
   // generowanie calego ssiedztwa
   for (int i = 0; i < iNrOfVar; i++)
   {
      fMinValue = AlgParam.Variables.GetVarMin(i);
      fMaxValue = AlgParam.Variables.GetVarMax(i);
      fRange = (fMaxValue - fMinValue) * AlgParam.fRangeA;
      if (bTempOk)
         fLastValue = Solution.GetLastSolVar(i);
      else
         fLastValue = newvals[i];

      int pos = pow(2, i);
      // sprawdzamy czy na okreslonej pozycji jest 1
      if (sol & pos)
      {
         int ii = 0;
         float fNewPart, fRand = fRange;
         for (ii; fRand < 100; ii++) fRand *= 10;
         fNewPart = (float)(rand() % (int)fRand);
         for (ii; ii > 0; ii--) fNewPart /= 10;

         fNewValue = fLastValue + fNewPart;
         if (fNewValue > fMaxValue)
            fNewValue = fLastValue;
      }
      else
      {
         int ii = 0;
         float fNewPart, fRand = fRange;
         for (ii; fRand < 100; ii++) fRand *= 10;
         fNewPart = (float)(rand() % (int)fRand);
         for (ii; ii > 0; ii--) fNewPart /= 10;

         fNewValue = fLastValue - fNewPart;
         if (fNewValue < fMinValue)
            fNewValue = fLastValue;
      }
      // zapisujaemy wartosc zmiennej
      vals[i] = fNewValue;
   }
}

//******************************************************************************

CSolution CLocalSearch::Greedy(CAlgParameters &AlgParam)
{
   bool bNewValue;
   double vals[MAXVAR];
   float fFuncValue, fNewFuncValue;
   std::vector <float> vNewSol;

   Solution.Clear();
   Solution = Initial(AlgParam);
   // Jezeli blad w parsowaniu funkcji
   if (Solution.GetNumberOfSol() == 0) return Solution;

   // OPTYMALIZACJA
   fFuncValue = Solution.GetLastFunc();
   int iNrOfVar = AlgParam.Variables.GetNumberOfVar();
   do
   {
      bNewValue = false;
      for (int sol = 0; sol < pow(iNrOfVar, 2); sol++)
      {
         // wyliczenie sasiedztwa dla kolejnego rozwiazania
         Neighbourhood(AlgParam, Solution, sol, vals);
         // wyliczenie funkcji celu
         fNewFuncValue = AlgParam.FuncParser.Eval(vals);
         vNewSol.clear();
         if (((!AlgParam.OptimMax) && (fFuncValue > fNewFuncValue)) || // minimalizacja
             ((AlgParam.OptimMax) && (fFuncValue < fNewFuncValue)))    // maksymalizacja
         {
            for (int i = 0; i < iNrOfVar; i++)
               vNewSol.push_back(vals[i]);
            Solution.Add(vNewSol, fNewFuncValue);
            Sleep(AlgParam.iLocalSleep);
            if (FormMain->bStop) return Solution;
            fFuncValue = fNewFuncValue;
            bNewValue = true;
            break;
         }
      }
   } while (bNewValue);
   return Solution;
}

//******************************************************************************
//******************************************************************************

CSolution CLocalSearch::Steepest(CAlgParameters &AlgParam)
{
   double vals[MAXVAR];
   bool bNewValue;
   float fFuncValue, fNewFuncValue, fGroupFuncValue;
   std::vector <float> vGroupNewSol;

   Solution.Clear();
   Solution = Initial(AlgParam);
   // Jezeli blad w parsowaniu funkcji
   if (Solution.GetNumberOfSol() == 0) return Solution;

   // OPTYMALIZACJA
   fFuncValue = Solution.GetLastFunc();
   fGroupFuncValue = fFuncValue;
   int iNrOfVar = AlgParam.Variables.GetNumberOfVar();
   do
   {
      bNewValue = false;
      for (int sol = 0; sol < pow(iNrOfVar, 2); sol++)
      {
         // wyliczenie sasiedztwa dla kolejnego rozwiazania
         Neighbourhood(AlgParam, Solution, sol, vals);
         // wyliczenie funkcji celu dla jednego rozwiazania w grupie
         fNewFuncValue = AlgParam.FuncParser.Eval(vals);
         if (((!AlgParam.OptimMax) && (fGroupFuncValue > fNewFuncValue)) || // minimalizacja
             ((AlgParam.OptimMax) && (fGroupFuncValue < fNewFuncValue)))    // maksymalizacja
         {
            vGroupNewSol.clear();
            for (int i = 0; i < iNrOfVar; i++)
               vGroupNewSol.push_back(vals[i]);
            fGroupFuncValue = fNewFuncValue;
         }
      }
      // rozwiazanie najlepsze do tej pory
      if (((!AlgParam.OptimMax) && (fFuncValue > fGroupFuncValue)) || // minimalizacja
          ((AlgParam.OptimMax) && (fFuncValue < fGroupFuncValue)))    // maksymalizacja
      {
         Solution.Add(vGroupNewSol, fGroupFuncValue);
         Sleep(AlgParam.iLocalSleep);
         if (FormMain->bStop) return Solution;
         fFuncValue = fGroupFuncValue;
         bNewValue = true;
      }
   } while (bNewValue);
   return Solution;
}

//////////////////////////////////////////////////////////////////////
CSolution CLocalSearch::SimulatedAnnealing(CAlgParameters &AlgParam)
{
   std::vector <float> vNewSol;
   double vals[MAXVAR], newvals[MAXVAR];
   float fFuncValue, fNewFuncValue;

	bool bZmianaTemperatury, bKoniecPoszukiwania, bTempUstalona;
	long int iRozw, iIleRozw = 0;
	long int iIleRozwAkcept = 0;
	long int iIleRozwNieAkcept = 0;;
	double Temperat, TEMP0 = (double)AlgParam.iTemp0;
	int Lk;
	double alfa = AlgParam.fAlfa;  // 0.8 < alfa <0.99
	long int stop = 0;

   // Inicjalizacja
   Solution.Clear();
   Solution = Initial(AlgParam);
   // Jezeli blad w parsowaniu funkcji
   if (Solution.GetNumberOfSol() == 0) return Solution;

   fFuncValue = Solution.GetLastFunc();
   int iNrOfVar = AlgParam.Variables.GetNumberOfVar();
   for (int ij = 0; ij < iNrOfVar; ij++)
      newvals[ij] = Solution.GetLastSolVar(ij);

	Temperat = TEMP0;
   Lk = pow(2, iNrOfVar) * AlgParam.iLk;   // ilosc iteracji na jednym poziomie temp.
	bKoniecPoszukiwania = false;
   bTempUstalona = false;

   // OPTYMALIZACJA
   if (iNrOfVar > 0)
	do // grzanie
	{
		stop = 1;
		iIleRozw = 0;
		iIleRozwAkcept = 0;
		iIleRozwNieAkcept = 0;
               //   bTempUstalona = true;
               //   Temperat=800;
      do // poszukiwanie
		{
		   bZmianaTemperatury = false;
		   iRozw = 0;
		   do // chlodzenie
		   {
            do
            {
               for (int sol = 0; ((sol < pow(iNrOfVar, 2))); sol++)
               {
                  iRozw++;
                  iIleRozw++;
                  // wyliczenie sasiedztwa dla kolejnego rozwiazania
                  NeighbourhoodSA(AlgParam, Solution, sol, vals, newvals, bTempUstalona);
                  // wyliczenie funkcji celu
                  fNewFuncValue = AlgParam.FuncParser.Eval(vals);

                  vNewSol.clear();
                  if (((!AlgParam.OptimMax) && (fFuncValue > fNewFuncValue)) || // minimalizacja
                      ((AlgParam.OptimMax) && (fFuncValue < fNewFuncValue)))    // maksymalizacja
                  {
                     fFuncValue = fNewFuncValue;
                     if (bTempUstalona)
                     {
                        for (int i = 0; i < iNrOfVar; i++)
                           vNewSol.push_back(vals[i]);
                        Solution.Add(vNewSol, fNewFuncValue);
                        Sleep(AlgParam.iLocalSleep);
                        if (FormMain->bStop) return Solution;
                     }
                     else
                     {
                        for (int ij = 0; ij < iNrOfVar; ij++)
                           newvals[ij] = vals[ij];
                        iIleRozwAkcept++;
                     }
                  }
                  else
                  {
                    if (!bTempUstalona)
                    {
           		  		double ex;
                     if (!AlgParam.OptimMax)
                        ex = exp((fFuncValue - fNewFuncValue) / Temperat * 10000);
                     if (AlgParam.OptimMax)
                        ex = (exp((fNewFuncValue - fFuncValue) / Temperat * 10000));
	           			double er = ((double)(rand() % 100) / 100) /*+ AlgParam.fEx / 100*/;
		     	      	if (ex > er)
      			     	{
                        fFuncValue = fNewFuncValue;
                        if (bTempUstalona)
                        {
                           for (int i = 0; i < iNrOfVar; i++)
                              vNewSol.push_back(vals[i]);
                           Solution.Add(vNewSol, fNewFuncValue);
                           Sleep(AlgParam.iLocalSleep);
                           if (FormMain->bStop) return Solution;
                        }
                        else
                        {
                           for (int ij = 0; ij < iNrOfVar; ij++)
                              newvals[ij] = vals[ij];
                           iIleRozwAkcept++;
                        }
   		         	}
                     else
                     {
                        if (!bTempUstalona)
                           iIleRozwNieAkcept++;
                     }
                    }
                  }
	   		      if (iRozw == Lk) //limit poszukiwan dla biezacej temperatury
      		   	{
		      	   	bZmianaTemperatury = true;
				      	break;
      				}

			      	if (bTempUstalona && (abs(fFuncValue - fNewFuncValue) < AlgParam.fStop))
      					stop++;
		      		else
				      	stop = 1;
      				if (stop >= Lk) //warunek stopu dla algorytmu
		      		{
				      	bKoniecPoszukiwania = true;
      					bZmianaTemperatury = true;
		      			break;
				      }
               } // koniec for(sol)
            } while (!bZmianaTemperatury);
   			if (!bTempUstalona) // jeden obieg petli
	   		{
		   		if ((((double)iIleRozwAkcept / (double)iIleRozw) * 100) > AlgParam.iPercentAccepted)
               {
			   		bTempUstalona = true;
                  //Przywrocenie inicjalizacji
                  fFuncValue = Solution.GetLastFunc();
                  for (int ij = 0; ij < iNrOfVar; ij++)
                     newvals[ij] = Solution.GetLastSolVar(ij);
               }
				   break;
   			}
	   		//chodzenie
		   	Temperat *= alfa;
            if (Temperat < 1)
            {
		      	bKoniecPoszukiwania = true;
   		 	   break;
            }
	   	} while (!bZmianaTemperatury);
     	   if (!bTempUstalona)
	         break;
		} while (!bKoniecPoszukiwania);
		// podgrzewanie
		Temperat += AlgParam.iTempPlus;
	} while (!bKoniecPoszukiwania);
   return Solution;
}
//---------------------------------------------------------------------------

CSolution CLocalSearch::GetSolution(void)
{
   return Solution;
}
