//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <cstring>
#include "MainForm.h"
#include "AboutForm.h"
#include "HelpFunctionForm.h"

#define crRotateCursor 100
#define crMoveCursor   101
#define crScaleCursor  102

#define ROTATE 1
#define MOVE   2
#define SCALE  3

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

using namespace std;
TFormMain *FormMain;

double Spiral(const double *p)
{
   return p[1] / 8 - 0.5;
}

void AdjustDelimeters(char* sFunction)
{
      for(unsigned int i = 0; i < strlen(sFunction); i++)
      {
         if (sFunction[i] == ',') sFunction[i] = '.';
         if (sFunction[i] == ';') sFunction[i] = ',';
      }
}

void AdjustDelimetersGrid(char* sFunction)
{
      for(unsigned int i = 0; i < strlen(sFunction); i++)
      {
         if (sFunction[i] == '.') sFunction[i] = ',';
      }
}
//---------------------------------------------------------------------------
__fastcall TFormMain::TFormMain(TComponent* Owner)
   : TForm(Owner)
{
   ChartReady = false;
   bResultsOpen = false;
   bStop = false;
   bSpiral = false;
   bThreadSuspend = false;
   bNewSolutions = false;
   kx = -90;
   ky = 0;
   PosX = -50;
   PosY = -900;
   Screen->Cursors[crRotateCursor] = LoadCursor((void *)HInstance, "ID_CUR_ROTATE");
   Screen->Cursors[crMoveCursor]  = LoadCursor((void *)HInstance, "ID_CUR_MOVE");
   Screen->Cursors[crScaleCursor] = LoadCursor((void *)HInstance, "ID_CUR_SCALE");
   LeftButtonAction = ROTATE;
   RightButtonAction = MOVE;
   LabelRotateLeft->Visible = true;
   LabelMoveRight->Visible = true;
   OpenDialog->InitialDir = GetCurrentDir();
   SaveDialog->InitialDir = GetCurrentDir();
   EditFunctionName->Text= "Example 1";
   
   fp. AddFunction("spiral", Spiral, 2);
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::FormCreate(TObject *Sender)
{
   StringGridVariables->Cells[0][0] = "Name";
   StringGridVariables->Cells[1][0] = "Min";
   StringGridVariables->Cells[2][0] = "Max";
   StringGridVariables->Cells[3][0] = "Value";
   StringGridVariables->Cells[0][1] = "x";
   StringGridVariables->Cells[1][1] = "-50";
   StringGridVariables->Cells[2][1] = "50";
   StringGridVariables->Cells[3][1] = "0";
   StringGridVariables->Cells[0][2] = "y";
   StringGridVariables->Cells[1][2] = "-40";
   StringGridVariables->Cells[2][2] = "40";
   StringGridVariables->Cells[3][2] = "6";
   StringGridVariables->Cells[0][3] = "c";
   StringGridVariables->Cells[1][3] = "1";
   StringGridVariables->Cells[2][3] = "15";
   StringGridVariables->Cells[3][3] = "5";

   /**/
   // tworzymy wykres
   hDC = GetDC(PanelChart->Handle);   //pobieramy DC - Device Context
   Chart = new COpenGLView(hDC);
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxWireframeClick(TObject *Sender)
{
   Redraw();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormPaint(TObject *Sender)
{
   Redraw();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormResize(TObject *Sender)
{
   if (PageControlMain->ActivePage == TabSheetChart)
   {
      Redraw();
      GroupBoxRangeMin->Width = (TabSheetChart->Width - GroupBoxRangeMin->Left - 8) / 2;
      GroupBoxRangeMax->Width = GroupBoxRangeMin->Width;
      GroupBoxRangeMax->Left = GroupBoxRangeMin->Width + GroupBoxRangeMin->Left - 2;
      EditMinX->Left = GroupBoxRangeMin->Width - 42;
      EditMinY->Left = GroupBoxRangeMin->Width - 42;
      EditMaxX->Left = GroupBoxRangeMax->Width - 42;
      EditMaxY->Left = GroupBoxRangeMax->Width - 42;
      ScrollBarMaxX->Width = ScrollBarMinX->Width;
      ScrollBarMaxY->Width = ScrollBarMinX->Width;
      ScrollBarMaxX->Left = ScrollBarMinX->Left;
      ScrollBarMaxY->Left = ScrollBarMinX->Left;
   }
   if (PageControlMain->ActivePage == TabSheetFunction)
   {
      StringGridVariables->Width = TabSheetFunction->Width - 60;
      StringGridVariables->Height = TabSheetFunction->Height - 250;
      StringGridVariables->RowCount = 20;
      StringGridVariables->ColCount = 4;
      StringGridVariables->DefaultColWidth = (StringGridVariables->Width - 30) / 4;
   }
  if(PageControlMain->ActivePage == TabSheetSolutions)
  {
      StringGridSolutions->Width = TabSheetSolutions->Width - 40;
      StringGridSolutions->Height = TabSheetSolutions->Height - 90;
      StringGridSolutions->RowCount = Solution.GetNumberOfSol() + 1;
      StringGridSolutions->ColCount = AlgParam.Variables.GetNumberOfVar() + 2;
      StringGridSolutions->DefaultColWidth = (TabSheetSolutions->Width - 75) /
                                    (AlgParam.Variables.GetNumberOfVar() + 2);
   }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxPolygonClick(TObject *Sender)
{
   Redraw();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxShadingClick(TObject *Sender)
{
   Redraw();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::CheckBoxColorsClick(TObject *Sender)
{
   Redraw();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::PageControlMainMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   Redraw();
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::ScrollBarScaleChange(TObject *Sender)
{
   Redraw();
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::Image_positionMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   if (Button == mbLeft)
   {
      LeftButtonAction = MOVE;
      LabelRotateLeft->Visible = false;
      LabelScaleLeft->Visible = false;
      LabelMoveLeft->Visible = true;
   }
   if (Button == mbRight)
   {
      RightButtonAction = MOVE;
      LabelRotateRight->Visible = false;
      LabelScaleRight->Visible = false;
      LabelMoveRight->Visible = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::Image_zoomMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   if (Button == mbLeft)
   {
      LeftButtonAction = SCALE;
      LabelRotateLeft->Visible = false;
      LabelScaleLeft->Visible = true;
      LabelMoveLeft->Visible = false;
   }
   if (Button == mbRight)
   {
      RightButtonAction = SCALE;
      LabelRotateRight->Visible = false;
      LabelScaleRight->Visible = true;
      LabelMoveRight->Visible = false;
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::Image_rotateMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   if (Button == mbLeft)
   {
      LeftButtonAction = ROTATE;
      LabelRotateLeft->Visible = true;
      LabelScaleLeft->Visible = false;
      LabelMoveLeft->Visible = false;
   }
   if (Button == mbRight)
   {
      RightButtonAction = ROTATE;
      LabelRotateRight->Visible = true;
      LabelScaleRight->Visible = false;
      LabelMoveRight->Visible = false;
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::PanelChartMouseMove(TObject *Sender, TShiftState Shift,
      int X, int Y)
{
   if (!ChartReady)
      return;

   float fZoom = ScrollBarScale->Position;
   //zoom
   if ((LeftButton && (LeftButtonAction == SCALE)) || (RightButton && (RightButtonAction == SCALE)))
   {

      while (LastY < Y)
      {
         fZoom /= 1.01;
         LastY++;
      }
      while (LastY>Y)
      {
         LastY--;
         fZoom *= 1.01;
      }
      if (fZoom < 100) fZoom = 100;
      if (fZoom > 10000) fZoom = 10000;
      ScrollBarScale->Position = fZoom;
   }

   //rotacja
   if ((LeftButton && (LeftButtonAction == ROTATE)) || (RightButton && (RightButtonAction == ROTATE)))
   {
   //rotacja X
   while (LastX < X)
   {
      LastX++;
      ky += 0.5;
   }
   while (LastX > X)
   {
      LastX--;
      ky -= 0.5;
   }
   while (ky < 0) ky += 360;
   while (ky > 360) ky -= 360;

   //rotacja Y
   while (LastY < Y)
   {
      LastY++;
      kx += 0.3;
   }
   while (LastY>Y)
   {
      LastY--;
      kx -= 0.3;
   }
   while (kx < 0) kx += 360;
   while (kx > 360) kx -= 360;
 }

  //przesuwanie
  if ((LeftButton && (LeftButtonAction == MOVE)) || (RightButton && (RightButtonAction == MOVE)))
  {
      //os X
      while (LastX < X)
      {
         LastX++;
         PosX += 10;
      }
      while (LastX > X)
      {
         LastX--;
         PosX -= 10;
      }
      //os Y
      while (LastY < Y)
      {
         LastY++;
         PosY -= 10;
      }
      while (LastY > Y)
      {
         LastY--;
         PosY += 10;
      }
   }
 //  Redraw();
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::PanelChartMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
   LastY = Y;    //zapamietujemy gdzie jest mysz
   LastX = X;    //w momencie naciskania przycisku

   if ((Button == mbLeft) || (Button == mbMiddle))
   {
      LeftButton = true;
      if (LeftButtonAction == ROTATE) PanelChart->Cursor = TCursor(crRotateCursor);
      if (LeftButtonAction == MOVE) PanelChart->Cursor = TCursor(crMoveCursor);
      if (LeftButtonAction == SCALE) PanelChart->Cursor = TCursor(crScaleCursor);
   }
   if (Button == mbRight)
   {
      RightButton = true;
      if (RightButtonAction == ROTATE) PanelChart->Cursor = TCursor(crRotateCursor);
      if (RightButtonAction == MOVE) PanelChart->Cursor = TCursor(crMoveCursor);
      if (RightButtonAction == SCALE) PanelChart->Cursor = TCursor(crScaleCursor);
   }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::PanelChartMouseUp(TObject *Sender, TMouseButton Button,
      TShiftState Shift, int X, int Y)
{
   if ((Button == 0) || (Button == 2))
   {
      LeftButton = false;
      PanelChart->Cursor = TCursor(crDefault);
   }
   if (Button == 1)
   {
      RightButton = false;
      PanelChart->Cursor = TCursor(crDefault);
   }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::Zamknij1Click(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::SpeedButtonCalcClick(TObject *Sender)
{
   PrepareData(true);
}
//---------------------------------------------------------------------------
int TFormMain::PrepareData(bool bOnlyData)
{
   // przygotowanie danych wykresu
   double dVals[MAXVAR];
   char sVars[MAXFUNCLEN];

   AlgParam.Variables.Clear();
   strcpy(sVars, "");
   int iIndexComboY = ComboBoxAxeY->ItemIndex;
   ComboBoxAxeX->Items->Clear();
   ComboBoxAxeY->Items->Clear();
   // wczytanie zmiennych funkcji
   char sFunctions[200];
   strcpy(sFunctions, EditFunction->Text.c_str());
   char* as = strstr(sFunctions, "spiral");
   if (as != NULL)
   {
      char* bs = strtok(as, "()+-=*/^%! |&<>;");
      if (bs != NULL)
         if (strcmp(bs, "spiral") == 0)
            bSpiral = true;
         else
            bSpiral = false;
   }
   for (int i = 1; i < StringGridVariables->RowCount; i++)
   {
      char sFunction[200];
      strcpy(sFunction, EditFunction->Text.c_str());
      char* a = strstr(sFunction, StringGridVariables->Cells[0][i].c_str()); // jezeli zmienna wystepuje w funkcji
      if (a == NULL) continue;
      char* b = strtok(a, "()+-=*/^%! |&<>;"); // wycinamy zmienn jako pojedynczy token
      if (b == NULL) continue;
      if (strcmp(b, StringGridVariables->Cells[0][i].c_str()) == 0) // jezeli zmienna istnieje
      {
         if ((StringGridVariables->Cells[1][i] != "") &&
             (StringGridVariables->Cells[2][i] != "") &&
             (StringGridVariables->Cells[3][i] != ""))
         {

            char smin[50];
            char smax[50];
            char sval[50];
            strcpy(smin, StringGridVariables->Cells[1][i].c_str());
            strcpy(smax, StringGridVariables->Cells[2][i].c_str());
            strcpy(sval, StringGridVariables->Cells[3][i].c_str());
            AdjustDelimetersGrid(smin);
            AdjustDelimetersGrid(smax);
            AdjustDelimetersGrid(sval);
            AlgParam.Variables.AddVariable(StringGridVariables->Cells[0][i].c_str(),
                                           StrToFloat(smin), StrToFloat(smax), StrToFloat(sval));
         }
         else
         {
            ShowMessage("Niekompletne dane wejsciowe !");
         }
      }
   }
   // Ustawienie ComboBoksow
   ComboBoxAxeY->Items->Add("-");
   for (int i = 0; i < AlgParam.Variables.GetNumberOfVar(); i++)
   {
      strcat(sVars, AlgParam.Variables.GetVarName(i));
      if (i != AlgParam.Variables.GetNumberOfVar() - 1)
         strcat(sVars, ",");
      if (bOnlyData)
         dVals[i] = StrToFloat(AlgParam.Variables.GetVarValue(i));
      if (AlgParam.Variables.GetNumberOfVar() == 1)
      {
         ComboBoxAxeX->Items->Add(AlgParam.Variables.GetVarName(i));
      }
      else
      {
         ComboBoxAxeX->Items->Add(AlgParam.Variables.GetVarName(i));
         ComboBoxAxeY->Items->Add(AlgParam.Variables.GetVarName(i));
      }
   }
   //przywrocenie indeksu ustawionego wczesniej
   if ((ComboBoxAxeY->Items->Count > iIndexComboY) && (iIndexComboY != -1))
      ComboBoxAxeY->ItemIndex = iIndexComboY;

   // parsowanie funkcji
   char sFunction[200];
   strcpy(sFunction, EditFunction->Text.c_str());
   AdjustDelimeters(sFunction);
   int ret = fp.Parse(sFunction, sVars);
   if(ret >= 0)
   {
       ShowMessage("Bld w funkcji. Kolumna: " + IntToStr(ret) +" : " + fp.ErrorMsg());
       return -1;
   }
   if (sLastFunction != sFunction)
   {
      Solution.Clear();
      Population.Clear();
   }
   sLastFunction = sFunction;
   // ustawienie parametrow algorytmu
   AlgParam.SetFunction(sFunction);
   AlgParam.SetFuncVars(sVars);
   AlgParam.OptimMax = RadioButton_max->Checked;
   /////////////////////////////////////////////
   AlgParam.fRangeA = StrToFloat(EditA->Text);
   AlgParam.iTemp0 = StrToInt(EditTemp0->Text);
   AlgParam.iTempPlus = StrToInt(EditTempPlus->Text);
   AlgParam.iPercentAccepted = StrToInt(EditPercentAc->Text);
   AlgParam.iLk = StrToInt(EditLk->Text);
   AlgParam.fAlfa = StrToFloat(EditAlfa->Text);
   AlgParam.fStop = StrToFloat(EditStop->Text);
   AlgParam.fEx = StrToFloat(EditEx->Text);
   /////////////////////////////////////////////
   AlgParam.iPopSize = StrToInt(EditPopSize->Text);
   AlgParam.iAlgIterat = StrToInt(EditIterations->Text);
   AlgParam.iTournamentSize = StrToInt(EditTournamentSize->Text);
   AlgParam.fCOProbability = StrToFloat(EditCOProbab->Text);
   AlgParam.fCOA = StrToFloat(EditCOA->Text);
   AlgParam.fMProbability = StrToFloat(EditMProbab->Text);
   AlgParam.fRangeMutA = StrToFloat(EditMutA->Text);
   if (RadioButtonTournament->Checked) AlgParam.iSelectionType = 0;
   if (RadioButtonRoulette->Checked) AlgParam.iSelectionType = 1;
   if (RadioButton1Point->Checked) AlgParam.iCrossOverType = 0;
   if (RadioButton2Point->Checked) AlgParam.iCrossOverType = 1;
   if (RadioButtonArithmetic->Checked) AlgParam.iCrossOverType = 2;
   /////////////////////////////////////////////
   AlgParam.iLocalSleep = StrToInt(EditLocalSleep->Text);
   AlgParam.iGeneticSleep = StrToInt(EditGeneticSleep->Text);
   /////////////////////////////////////////////

   if (!bOnlyData)
      PrepareFunction();
   else
   {
      char Sol[50];
      sprintf(Sol, "%2.3g", fp.Eval(dVals));
      LabelFuncCalc->Caption = Sol;
   }
   return 0;
}

//---------------------------------------------------------------------------
void TFormMain::PrepareFunction(void)
{
   // przygotowanie danych wykresu
   bool bOneVar;

   // Przydzielenie zmiennych do osi x i y
   if (AlgParam.Variables.GetNumberOfVar() > 0)
   {
      if (ComboBoxAxeX->ItemIndex != -1)
         iFirstVar = ComboBoxAxeX->ItemIndex;
      else
      {
         iFirstVar = 0;
         ComboBoxAxeX->ItemIndex = 0;
      }
      if (ComboBoxAxeY->ItemIndex != -1)
         iSecondVar = ComboBoxAxeY->ItemIndex - 1;
      else
      {
         iSecondVar = 1;
         ComboBoxAxeY->ItemIndex = 2;
      }

      // Typ wykresu 1D czy 2D
      if ((AlgParam.Variables.GetNumberOfVar() == 1) || (ComboBoxAxeY->ItemIndex == 0))
      {
         // Ustawienie skroli
         SetScrolls(true, iFirstVar, iSecondVar);
         if (AlgParam.Variables.GetNumberOfVar() == 1)
         {
            ComboBoxAxeX->ItemIndex = 0;
            ComboBoxAxeY->ItemIndex = 0;
         }
         iFirstVar = ComboBoxAxeX->ItemIndex;
         CalcFunc2DValue(iFirstVar);
      }
      else
      {
         // Ustawienie skroli
         SetScrolls(false, iFirstVar, iSecondVar);
         CalcFunc3DValue(iFirstVar, iSecondVar);
      }
   }
   else // funkcja stala
   {
      double dVals[MAXVAR];
      float fFunVal;
      vFuncValues.clear();
      for(int y = 0; y < GRIDH; y++)
         for(int x = 0; x < GRIDW; x++)
         {
            fFunVal = fp.Eval(dVals);
            vFuncValues.push_back(fFunVal);
         }
      MinX = 0;
      MaxX = 0;
      MinY = 0;
      MaxY = 0;
      MinZ = 0;
      MaxZ = 0;
   }
}

//---------------------------------------------------------------------------
void TFormMain::SetScrolls(bool bOneVar, int iFirstVar, int iSecondVar)
{
   ScrollBarMinX->Min = AlgParam.Variables.GetVarMin(iFirstVar);
   ScrollBarMinX->Max = AlgParam.Variables.GetVarMax(iFirstVar);
   ScrollBarMinX->Position = ScrollBarMinX->Min;
   ScrollBarMaxX->Min = AlgParam.Variables.GetVarMin(iFirstVar);
   ScrollBarMaxX->Max = AlgParam.Variables.GetVarMax(iFirstVar);
   ScrollBarMaxX->Position = ScrollBarMaxX->Max;
   ScrollBarMinX->Enabled = true;
   ScrollBarMaxX->Enabled = true;
   EditMinX->Text = AlgParam.Variables.GetVarMin(iFirstVar);
   EditMaxX->Text = AlgParam.Variables.GetVarMax(iFirstVar);
   EditMinX->Color = clWhite;
   EditMaxX->Color = clWhite;
   EditMinX->Enabled = true;
   EditMaxX->Enabled = true;
   ScrollBarMinX->Enabled = true;
   ScrollBarMaxX->Enabled = true;

   if (bOneVar)
   {
      ScrollBarMinY->Enabled = false;
      ScrollBarMaxY->Enabled = false;
      EditMinY->Enabled = false;
      EditMaxY->Enabled = false;
      EditMinY->Color = clScrollBar;
      EditMaxY->Color = clScrollBar;
   }
   else
   {
      ScrollBarMinY->Enabled = true;
      ScrollBarMaxY->Enabled = true;
      EditMinY->Enabled = true;
      EditMaxY->Enabled = true;
      EditMinY->Color = clWhite;
      EditMaxY->Color = clWhite;
      ScrollBarMinY->Min = AlgParam.Variables.GetVarMin(iSecondVar);
      ScrollBarMinY->Max = AlgParam.Variables.GetVarMax(iSecondVar);
      ScrollBarMinY->Position = ScrollBarMinY->Min;
      ScrollBarMaxY->Min = AlgParam.Variables.GetVarMin(iSecondVar);
      ScrollBarMaxY->Max = AlgParam.Variables.GetVarMax(iSecondVar);
      ScrollBarMaxY->Position = ScrollBarMaxY->Max;
      EditMinY->Text = AlgParam.Variables.GetVarMin(iSecondVar);
      EditMaxY->Text = AlgParam.Variables.GetVarMax(iSecondVar);
   }
}
//---------------------------------------------------------------------------
void TFormMain::CalcFunc3DValue(int iFirstVar, int iSecondVar)
{
   double dVals[MAXVAR];
   float fTempMax, fTempMin, fFunVal;
   vFuncValues.clear();

   MinX = StrToFloat(EditMinX->Text);
   MaxX = StrToFloat(EditMaxX->Text);
   MinY = StrToFloat(EditMinY->Text);
   MaxY = StrToFloat(EditMaxY->Text);

   // Ustawienie wartosci zmiennych dla wyliczenia wartosci funkcji
   for (int i = 0; i < AlgParam.Variables.GetNumberOfVar(); i++)
      dVals[i] = AlgParam.Variables.GetVarValue(i);

   float fUnitX = (fabs(MaxX - MinX)) / GRIDW;
   float fUnitY = (fabs(MaxY - MinY)) / GRIDH;

   for(int y = 0; y < GRIDH; y++)
      for(int x = 0; x < GRIDW; x++)
      {
         dVals[iFirstVar] = (x * fUnitX) + MinX;
         dVals[iSecondVar] = (y * fUnitY) + MinY;
         fFunVal = fp.Eval(dVals);        // wyliczenie funkcji
         vFuncValues.push_back(fFunVal);

         if (fFunVal < fTempMin) fTempMin = fFunVal;
         if (fFunVal > fTempMax) fTempMax = fFunVal;
      }
   MaxZ = fTempMax;
   MinZ = fTempMin;
}

//---------------------------------------------------------------------------
void TFormMain::CalcFunc2DValue(int iFirstVar)
{
   double dVals[MAXVAR];
   float fTempMax, fTempMin, fFunVal;
   vFuncValues.clear();

   MinX = StrToFloat(EditMinX->Text);
   MaxX = StrToFloat(EditMaxX->Text);
   MinY = 0;
   MaxY = 0;

   // Ustawienie wartosci zmiennych dla wyliczenia wartosci funkcji
   for (int i = 0; i < AlgParam.Variables.GetNumberOfVar(); i++)
      dVals[i] = AlgParam.Variables.GetVarValue(i);
   float fUnitX = (fabs(MaxX - MinX)) / GRIDW;

   for(int x = 0; x < GRIDW; x++)
   {
      dVals[iFirstVar] = (x * fUnitX) + MinX;
      fFunVal = fp.Eval(dVals);        // wyliczenie funkcji
      vFuncValues.push_back(fFunVal);

      if (fFunVal < fTempMin) fTempMin = fFunVal;
      if (fFunVal > fTempMax) fTempMax = fFunVal;
   }
   MaxZ = fTempMax;
   MinZ = fTempMin;
}

//---------------------------------------------------------------------------
void __fastcall TFormMain::TimerChartTimer(TObject *Sender)
{
   std::vector <float> vSol;
   vSol.clear();
   std::vector <float> vGASol;
   vGASol.clear();

   // pobranie wynikow z watku
   if ((!bResultsOpen) && (ot != NULL))
   {
      Solution = ot->GetResults();
      Population = ot->GetPopulationGA();
   }
   // skasowanie watku
   if ((ot != NULL) && ot->GetState())
      ReleaseButtons();

   // Przygotowanie danych do wyswietlenia
   // Przepisanie rozwiazan wyliczonych w watku do tablicy jednowymiarowej
   if ((MaxY == 0) && (MinY == 0))
   {
      if ((MaxX == 0) && (MinX == 0))
      {
         if (Solution.GetNumberOfSol() > 0)
            vSol.push_back(Solution.GetFunc(0)); // z
         for (int ii = 0; ii < Population.GetNumberOfSol(); ii++)
            vGASol.push_back(Population.GetFunc(0)); // z
      }
      else
      {
         for (int ii = 0; ii < Solution.GetNumberOfSol(); ii++)
         {
            vSol.push_back(Solution.GetSolutionVar(ii, iFirstVar)); // x
            vSol.push_back(Solution.GetFunc(ii)); // z
         }
         for (int ii = 0; ii < Population.GetNumberOfSol(); ii++)
         {
            vGASol.push_back(Population.GetSolutionVar(ii, iFirstVar)); // x
            vGASol.push_back(Population.GetFunc(ii)); // z
         }
      }
   }
   else
   {
      for (int ii = 0; ii < Solution.GetNumberOfSol(); ii++)
      {
         vSol.push_back(Solution.GetSolutionVar(ii, iFirstVar)); // x
         vSol.push_back(Solution.GetSolutionVar(ii, iSecondVar)); // y
         vSol.push_back(Solution.GetFunc(ii)); // z
      }
      for (int ii = 0; ii < Population.GetNumberOfSol(); ii++)
      {
         vGASol.push_back(Population.GetSolutionVar(ii, iFirstVar)); // x
         vGASol.push_back(Population.GetSolutionVar(ii, iSecondVar)); // y
         vGASol.push_back(Population.GetFunc(ii)); // z
      }
   }
   //przekazanie danych dla wykresu
   if (PageControlMain->ActivePage == TabSheetChart)
   {
      Chart->SetVertexData(MinX, MaxX, MinY, MaxY, MinZ, MaxZ, vFuncValues, vSol, vGASol);
      ChartReady = true;
      Redraw();
   }
  if (PageControlMain->ActivePage == TabSheetSolutions)
    PageControlMainChange(Sender);
}
//---------------------------------------------------------------------------
void TFormMain::Redraw(void)
{
   if (ChartReady)
   {
      RECT PanelArea;
      PanelArea.left = 0;
      PanelArea.top = 0;
      PanelArea.right = PanelChart->Width;
      PanelArea.bottom = PanelChart->ClientHeight;

      Chart->Render(PanelArea, bDemo, ScrollBarScale->Position, kx, ky, PosX, PosY,
                    CheckBoxWireframe->Checked, CheckBoxPolygon->Checked,
                    CheckBoxShading->Checked, CheckBoxSolution->Checked,
                    CheckBoxAxes->Checked, CheckBoxAxesDesc->Checked,
                    CheckBoxFrames->Checked, bSpiral);
      SwapBuffers(hDC);            //zamiana ekranow
   }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::SpeedButtonGreedyClick(TObject *Sender)
{
   if (PrepareData() == -1) return;
   BlockButtons();
   bResultsOpen = false;
   ot = new OptimizationThread(false, 0, AlgParam);
   if (bDemo) bDemo = false;
    else TimerChart->Enabled = true;
   PageControlMain->ActivePage = TabSheetChart;
   StatusBar->SimpleText = "Greedy algorithm is running ...";
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::SpeedButtonSteepestClick(TObject *Sender)
{
   if (PrepareData() == -1) return;
   BlockButtons();
   bResultsOpen = false;
   ot = new OptimizationThread(false, 1, AlgParam);
   if (bDemo) bDemo = false;
   else TimerChart->Enabled = true;
   PageControlMain->ActivePage = TabSheetChart;
   StatusBar->SimpleText = "Steepest algorithm is running ...";
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::SpeedButtonSAClick(TObject *Sender)
{
   if (PrepareData() == -1) return;
   BlockButtons();
   bResultsOpen = false;
   ot = new OptimizationThread(false, 2, AlgParam);
   if (bDemo) bDemo = false;
   else TimerChart->Enabled = true;
   PageControlMain->ActivePage = TabSheetChart;
   StatusBar->SimpleText = "Simulated Annealing algorithm is running ...";
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::SpeedButtonGeneticAlgClick(TObject *Sender)
{
   if (PrepareData() == -1) return;
   BlockButtons();
   bResultsOpen = false;
   ot = new OptimizationThread(false, 3, AlgParam);
   if (bDemo) bDemo = false;
   else TimerChart->Enabled = true;
   PageControlMain->ActivePage = TabSheetChart;
   StatusBar->SimpleText = "Genetic algorithm is running ...";
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::ComboBoxAxeXChange(TObject *Sender)
{
   PrepareFunction();
   TimerChartTimer(Sender);
   Redraw();
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::PageControlMainChange(TObject *Sender)
{
   FormResize(Sender);
   if (PageControlMain->ActivePage == TabSheetFunction)
   {
      StringGridVariables->Width = TabSheetFunction->Width - 60;
      StringGridVariables->Height = TabSheetFunction->Height - 250;
      StringGridVariables->RowCount = 20;
      StringGridVariables->ColCount = 4;
      StringGridVariables->DefaultColWidth = (StringGridVariables->Width - 30) / 4;
   }
  if(PageControlMain->ActivePage == TabSheetChart)
  {
      if (bDemo) Demonstracja1Click(Sender);
      if (PrepareData() == -1) return;
      bResultsOpen = true;
      if (bDemo) bDemo = false;
      else TimerChart->Enabled = true;
  }
  if(PageControlMain->ActivePage == TabSheetSolutions)
  {
    StringGridSolutions->Width = TabSheetSolutions->Width - 40;
    StringGridSolutions->Height = TabSheetSolutions->Height - 90;
    StringGridSolutions->RowCount = Solution.GetNumberOfSol() + 1;
    StringGridSolutions->ColCount = AlgParam.Variables.GetNumberOfVar() + 2;
    StringGridSolutions->DefaultColWidth = (TabSheetSolutions->Width - 75) /
                                    (AlgParam.Variables.GetNumberOfVar() + 2);
   if (Solution.GetNumberOfSol() > 0)
   {
      float fResult;
      // nazwy zmiennych
      AnsiString sFunc = "F(";
      StringGridSolutions->Cells[0][0] = "Nr";
      for (int j = 1; j <= AlgParam.Variables.GetNumberOfVar(); j++)
      {
         StringGridSolutions->Cells[j][0] = AlgParam.Variables.GetVarName(j - 1);
         sFunc += AlgParam.Variables.GetVarName(j - 1);
         if (j != AlgParam.Variables.GetNumberOfVar())
            sFunc += ",";
      }
      StringGridSolutions->Cells[StringGridSolutions->ColCount - 1][0] = sFunc + ")";
      // wartosci zmiennych
      fResult = Solution.GetFunc(0);
      LabelBeforeOpt->Caption = FloatToStrF(fResult, ffGeneral, 7, 20);
      for (int i = 0; i < Solution.GetNumberOfSol(); i++)
      {
         StringGridSolutions->Cells[0][i + 1] = i + 1;
         for (int j = 1; j <= AlgParam.Variables.GetNumberOfVar(); j++)
            StringGridSolutions->Cells[j][Solution.GetNumberOfSol() - i] = FloatToStrF(Solution.GetSolutionVar(i, j - 1), ffGeneral, 6, 20);
         float fSol = Solution.GetFunc(i);
         StringGridSolutions->Cells[StringGridSolutions->ColCount - 1][Solution.GetNumberOfSol() - i] = FloatToStrF(fSol, ffGeneral, 6, 20);
         if (AlgParam.OptimMax && (fResult < fSol)) fResult = fSol;
         if (!AlgParam.OptimMax && (fResult > fSol)) fResult = fSol;
      }
      LabelAfterOpt->Caption = FloatToStrF(fResult, ffGeneral, 7, 20);
   }
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::SpeedButtonNewClick(TObject *Sender)
{
	if (bNewSolutions)
	{
		if (Application->MessageBox("Sure ? Solutions not saved !", "Confimation", MB_YESNO) == IDYES)
         bNewSolutions = false;
      else return;
   }
   bSpiral = false;
   kx = -90;
   ky = 0;
   PosX = -50;
   PosY = 0;
   EditFunction->Text = "";
   for(int i = 1; i < StringGridVariables->RowCount - 1; i++)
      for(int j = 0; j < StringGridVariables->ColCount; j++)
         StringGridVariables->Cells[j][i] = "";
   PageControlMain->ActivePage = TabSheetFunction;
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::SpeedButtonOpenClick(TObject *Sender)
{
   FILE *file;
   char s[500];
   char text[500];

	if (bNewSolutions)
	{
		if (Application->MessageBox("Sure ? Solutions not saved !", "Confimation", MB_YESNO) == IDYES)
         bNewSolutions = false;
      else return;
   }
   if (OpenDialog->Execute()==IDOK)
   {
      bSpiral = false;
      kx = -90;
      ky = 0;
      PosX = -50;
      PosY = -900;
      EditFunction->Text = "";
      for(int i = 1; i < StringGridVariables->RowCount - 1; i++)
      for(int j = 0; j < StringGridVariables->ColCount; j++)
         StringGridVariables->Cells[j][i] = "";
      // skasowanie watku
      if ((ot != NULL) && ot->GetState())
         ReleaseButtons();
      Solution.Clear();
      Population.Clear();
      file = fopen(OpenDialog->FileName.c_str(),"r");

      int a = 0;
	   do {
         a++;
         fscanf(file, "%20s", s);
         if (a > 10)
      	{
	         ShowMessage("Wrong file format !");
	         return;
   	   }
      } while (!(strcmp(s, "OptiVis") == 0));

      FormMain->Caption = "File: " + OpenDialog->FileName;
      //#
      fscanf(file, "%20s", s);
      // nazwa funkcji
      strcpy(text, "");
      fscanf(file, "%200s", s);
	   while (!(strcmp(s, "#") == 0))
      {
         strcat(text, s);
         strcat(text, " ");
         fscanf(file, "%200s", s);
      }
      EditFunctionName->Text = text;
      // Pominiecie naglowka
	   do {fscanf(file, "%200s", s);} while (!(strcmp(s, "formula") == 0));
      // Czytanie funkcji
      strcpy(text, "");
      fscanf(file, "%500s", s);
	   while (!(strcmp(s, "#") == 0))
      {
         strcat(text, s);
         fscanf(file, "%500s", s);
      }
      // pominiecie 'f='
      int i = 0;
      for (i; i < strlen(text); i++)
         if (i > 1) s[i-2] = text[i];
      s[i-2]='\0';

      for(unsigned int i = 0; i < strlen(s); i++)
         if (s[i] == ',') s[i] = '.';
      //      AdjustDelimeters(text);
      EditFunction->Text = s;

      // Czytanie kierunku optymalizacji
	   do {fscanf(file, "%50s", s);} while (!((strcmp(s, "max=0") == 0) || (strcmp(s, "max=1") == 0)));
      if (s[4] == '1')
         RadioButton_max->Checked = true;
      else
         RadioButton_min->Checked = true;

      // Czytanie liczby zmiennych
	   do {fscanf(file, "%50s", s);} while (!(strcmp(s, "variables") == 0));
      fscanf(file, "%200s", s);
      // pominiecie 'vars='
      strcpy(text, s);
      i = 0;
      for (i; i < strlen(text); i++)
         if (i > 4) s[i-5] = text[i];
      s[i-5]='\0';

      int iVar = StrToInt(s);

	   do {fscanf(file, "%50s", s);} while (!(strcmp(s, "default") == 0));
      AlgParam.Variables.Clear();
      float min, max, value;
      int j;
      for (int i = 1; i < iVar + 1; i++)
      {
      	fscanf(file," %200s", s);
         for (j = 0; j < strlen(s); j++)
            if (s[j] != ',') text[j] = s[j];
            else break;
         text[j] = '\0';
         StringGridVariables->Cells[0][i] = text;
         // pozostala czesc stringa
         j++;
         int k;
         for (k = j; k < strlen(s); k++)
            text[k - j] = s[k];
         text[k - j] = '\0';

      	sscanf(text,"%g,%g,%g", &min, &max, &value);
         StringGridVariables->Cells[1][i] = FloatToStrF(min, ffGeneral, 4, 20);
         StringGridVariables->Cells[2][i] = FloatToStrF(max, ffGeneral, 4, 20);
         StringGridVariables->Cells[3][i] = FloatToStrF(value, ffGeneral, 4, 20);
      }
      // Czytanie liczby rozwiazan
	   do {fscanf(file, "%50s", s);} while (!(strcmp(s, "solutions") == 0));
      fscanf(file, "%200s", s);
      // pominiecie 'sols='
      strcpy(text, s);
      i = 0;
      for (i; i < strlen(text); i++)
         if (i > 4) s[i-5] = text[i];
      s[i-5]='\0';

      int iSol = StrToInt(s);

      if (iSol > 0)
	      do {fscanf(file, "%50s", s);} while (!(strcmp(s, "variables") == 0));
      vVecData1D vPartFuncValues;
      Solution.Clear();
      float f;
      int ik;
      for (int i = 0; i < iSol; i++)
      {
         vPartFuncValues.clear();
    	   for (int j = 0; j < iVar; j++)
         {
      	   fscanf(file," %g,", &f);
            vPartFuncValues.push_back(f);
         }
         fscanf(file," %g ", &f);
         Solution.Add(vPartFuncValues, f);
      }
      fclose(file);

      // parsowanie funkcji
      char sFunction[200];
      strcpy(sFunction, EditFunction->Text.c_str());
      AdjustDelimeters(sFunction);
      sLastFunction = sFunction;
      
      if (PrepareData() == -1) return;
      bResultsOpen = true;
      bNewSolutions = false;
      TimerChart->Enabled = true;
   }
   else
      bResultsOpen = false;

   // skasowanie watku
   if ((ot != NULL) && ot->GetState())
      ReleaseButtons();      
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::SpeedButtonSaveClick(TObject *Sender)
{
   if (PrepareData(true) == -1) return;
   FILE *file;
   char text[255];
   char ss[50];
   int max;
   if (SaveDialog->Execute() == IDOK)
   {
      if (RadioButton_max->Checked) max = 1;
      else max = 0;

      file = fopen(SaveDialog->FileName.c_str(),"w+");
      fprintf(file, "# data file for OptiVis\n");
      fprintf(file, "# %s\n\n", EditFunctionName->Text);

      char sFunction[200];
      strcpy(sFunction, EditFunction->Text.c_str());
      for(unsigned int i = 0; i < strlen(sFunction); i++)
      {
         if (sFunction[i] == ',') sFunction[i] = '.';
      }
      fprintf(file, "# formula\nf=%s\n\n", sFunction);
      fprintf(file, "# 1 for maximization, 0 for minimization\nmax=%i \n\n", max);
      fprintf(file, "# number of variables\nvars=%i \n\n", AlgParam.Variables.GetNumberOfVar());
      fprintf(file, "# variables: name, min, max, default\n");
      if (AlgParam.Variables.GetNumberOfVar() > 0)
      {
   	   for (int i = 0; i < AlgParam.Variables.GetNumberOfVar(); i++)
            fprintf(file, "%s,%lg,%lg,%lg\n", AlgParam.Variables.GetVarName(i),
                    AlgParam.Variables.GetVarMin(i), AlgParam.Variables.GetVarMax(i), AlgParam.Variables.GetVarValue(i));
      }
      fprintf(file, "\n# number of solutions\nsols=%i\n\n", Solution.GetNumberOfSol());
      if (Solution.GetNumberOfSol() > 0)
      {
         fprintf(file, "# solutions - comma-delimited values list for variables and value of function (last)\n");
   	   for (int i = 0; i < Solution.GetNumberOfSol(); i++)
         {
      	   for (int j = 0; j < AlgParam.Variables.GetNumberOfVar(); j++)
               fprintf(file, "%lg,", Solution.GetSolutionVar(i, j));
            fprintf(file, "%lg\n", Solution.GetFunc(i));
         }
      }
      bNewSolutions = false;
      fclose(file);
   }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::Funkcja1Click(TObject *Sender)
{
   PageControlMain->ActivePage = TabSheetFunction;
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::Rozwizania1Click(TObject *Sender)
{
   PageControlMain->ActivePage = TabSheetSolutions;
   FormResize(Sender);
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::Oprogramie1Click(TObject *Sender)
{
   FormAbout->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::Algorytmylokalne2Click(TObject *Sender)
{
   PageControlMain->ActivePage = TabSheetOptions;
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::AlgorytmGenetyczny2Click(TObject *Sender)
{
   PageControlMain->ActivePage = TabSheetOptionsGenetic;
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::SpeedButtonStopClick(TObject *Sender)
{
   ReleaseButtons();
}
//---------------------------------------------------------------------------

void TFormMain::BlockButtons(void)
{
   SpeedButtonNew->Enabled = false;
   SpeedButtonOpen->Enabled = false;
   SpeedButtonSave->Enabled = false;
   Nowafunkcja1->Enabled = false;
   Wczytajwyniki1->Enabled = false;
   Zapiszwyniki1->Enabled = false;
   Greedy1->Enabled = false;
   Steepest1->Enabled = false;
   Symulowanewyarzanie1->Enabled = false;
   AlgorytmGenetyczny1->Enabled = false;
   Demonstracja1->Enabled = false;

   SpeedButtonGreedy->Enabled = false;
   SpeedButtonSteepest->Enabled = false;
   SpeedButtonSA->Enabled = false;
   SpeedButtonGeneticAlg->Enabled = false;
   StringGridVariables->Enabled = false;
   EditFunction->ReadOnly = true;
   SpeedButtonPause->Down = false;
   Pauza1->Checked = false;
   bStop = false;
   bNewSolutions = true;
}

void TFormMain::ReleaseButtons(void)
{
   bStop = true;
   SpeedButtonNew->Enabled = true;
   SpeedButtonOpen->Enabled = true;
   SpeedButtonSave->Enabled = true;
   Nowafunkcja1->Enabled = true;
   Wczytajwyniki1->Enabled = true;
   Zapiszwyniki1->Enabled = true;
   Greedy1->Enabled = true;
   Steepest1->Enabled = true;
   Symulowanewyarzanie1->Enabled = true;
   AlgorytmGenetyczny1->Enabled = true;
   Demonstracja1->Enabled = true;

   SpeedButtonGreedy->Enabled = true;
   SpeedButtonSteepest->Enabled = true;
   StringGridVariables->Enabled = true;
   SpeedButtonSA->Enabled = true;
   SpeedButtonGeneticAlg->Enabled = true;
   EditFunction->ReadOnly = false;
   Pauza1->Checked = false;
   bThreadSuspend = false;
   SpeedButtonPause->Down = false;
   SpeedButtonPause->Hint="Pause calculation";
   StatusBar->SimpleText = "";
   if (ot != NULL)
   {
      ot->FreeOnTerminate = true;
      ot->Terminate();
      ot = NULL;
   }
}

void __fastcall TFormMain::ScrollBarMinXChange(TObject *Sender)
{
   if (ScrollBarMinX->Position >= ScrollBarMaxX->Position)
      ScrollBarMinX->Position = ScrollBarMaxX->Position - 1;
   EditMinX->Text = IntToStr(ScrollBarMinX->Position);
   CalcFunc3DValue(iFirstVar, iSecondVar);
   Redraw();
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::ScrollBarMinYChange(TObject *Sender)
{
   if (ScrollBarMinY->Position >= ScrollBarMaxY->Position)
      ScrollBarMinY->Position = ScrollBarMaxY->Position - 1;
   EditMinY->Text = IntToStr(ScrollBarMinY->Position);
   CalcFunc3DValue(iFirstVar, iSecondVar);
   Redraw();
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::ScrollBarMaxXChange(TObject *Sender)
{
   if (ScrollBarMaxX->Position <= ScrollBarMinX->Position)
      ScrollBarMaxX->Position = ScrollBarMinX->Position + 1;
   EditMaxX->Text = IntToStr(ScrollBarMaxX->Position);
   CalcFunc3DValue(iFirstVar, iSecondVar);
   Redraw();
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::ScrollBarMaxYChange(TObject *Sender)
{
   if (ScrollBarMaxY->Position <= ScrollBarMinY->Position)
      ScrollBarMaxY->Position = ScrollBarMinY->Position + 1;
   EditMaxY->Text = IntToStr(ScrollBarMaxY->Position);
   CalcFunc3DValue(iFirstVar, iSecondVar);
   Redraw();
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::Demonstracja1Click(TObject *Sender)
{
   if(!Demonstracja1->Checked)
   {
      bDemo = true;
      TimerChart->Enabled = false;
      ChartReady = true;
      PageControlMain->ActivePage = TabSheetChart;
      Demonstracja1->Checked = true;
      TimerDemo->Enabled = true;
      ToolBarMain->Enabled = false;
   }
   else
   {
      Demonstracja1->Checked = false;
      bDemo = false;
      TimerDemo->Enabled = false;
      PageControlMainChange(Sender);
      ToolBarMain->Enabled = true;
   }
}
//---------------------------------------------------------------------------


void __fastcall TFormMain::FormClose(TObject *Sender, TCloseAction &Action)
{
   if (bDemo) bDemo = false;
	if (bNewSolutions)      
	{
		if (Application->MessageBox("Really close ?", "Closing program", MB_YESNO) == IDYES)
		   Action = caFree;
	   else Action = caNone;
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::TimerAutoRotationTimer(TObject *Sender)
{
  if(PageControlMain->ActivePage == TabSheetChart)
  {
   if (CheckBoxAutoRotation->Checked && ChartReady)
   {
      ky -= TrackBarAutoRotation->Position * 0.1;
      if (ky < 0)
         ky += 360;
   //   Redraw();
   }
  }
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::Wykresfunkcji1Click(TObject *Sender)
{
   PageControlMain->ActivePage = TabSheetChart;
}
//---------------------------------------------------------------------------


void __fastcall TFormMain::Pomoc2Click(TObject *Sender)
{
   FormHelpFunction->Show();
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::TimerDemoTimer(TObject *Sender)
{
  if(PageControlMain->ActivePage == TabSheetChart)
  {
     Redraw();
  }
}
//---------------------------------------------------------------------------


void __fastcall TFormMain::TrackBarIterationsChange(TObject *Sender)
{
   EditIterations->Text = IntToStr(TrackBarIterations->Position);
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::TrackBarPopSizeChange(TObject *Sender)
{
   EditPopSize->Text = IntToStr(TrackBarPopSize->Position);
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::TrackBarGeneticSleepChange(TObject *Sender)
{
   EditGeneticSleep->Text = IntToStr(TrackBarGeneticSleep->Position);
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::TrackBarTournamentSizeChange(TObject *Sender)
{
   EditTournamentSize->Text = IntToStr(TrackBarTournamentSize->Position);
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::TrackBarCOAChange(TObject *Sender)
{
   EditCOA->Text = FloatToStr((float)TrackBarCOA->Position/100);   
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::TrackBarCOProbabChange(TObject *Sender)
{
   EditCOProbab->Text = IntToStr(TrackBarCOProbab->Position);
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::TrackBarMutAChange(TObject *Sender)
{
   EditMutA->Text = FloatToStr((float)TrackBarMutA->Position/1000);
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::TrackBarMProbabChange(TObject *Sender)
{
   EditMProbab->Text = IntToStr(TrackBarMProbab->Position);
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::BitBtnResetClick(TObject *Sender)
{
   bSpiral = false;
   kx = -90;
   ky = 0;
   PosX = -50;
   PosY = -900;
   CheckBoxPolygon->Checked = true;
   CheckBoxFrames->Checked = false;
   CheckBoxWireframe->Checked = false;
   ScrollBarScale->Position = 4000;
   LeftButtonAction = ROTATE;
   RightButtonAction = MOVE;
   LabelRotateLeft->Visible = true;
   LabelMoveRight->Visible = true;
   LabelScaleLeft->Visible = false;
   LabelMoveLeft->Visible = false;
   LabelRotateRight->Visible = false;
   LabelScaleRight->Visible = false;
   Redraw();
}
//---------------------------------------------------------------------------


void __fastcall TFormMain::SpeedButtonPauseClick(TObject *Sender)
{
   if(SpeedButtonPause->Down)
   {
      bThreadSuspend = true;
      SpeedButtonPause->Hint="Continue calculation";
      Pauza1->Checked = true;
   }
   else
   {
      bThreadSuspend = false;
      SpeedButtonPause->Hint="Pause calculation";
      Pauza1->Checked = false;
   }
   ThreadSter();
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::Pauza1Click(TObject *Sender)
{
   if(!Pauza1->Checked)
   {
      bThreadSuspend = true;
      SpeedButtonPause->Hint="Kontynuj obliczenia";
      SpeedButtonPause->Down = true;
      Pauza1->Checked = true;
   }
   else
   {
      Pauza1->Checked = false;
      bThreadSuspend = false;
      SpeedButtonPause->Down = false;
      SpeedButtonPause->Hint="Wstrzymaj obliczenia";
   }
   ThreadSter();
}
//---------------------------------------------------------------------------

void TFormMain::ThreadSter(void)
{
   if (ot != NULL)
      if (bThreadSuspend)
         ot->Suspend();
      else
         ot->Resume();
}
void __fastcall TFormMain::TrackBarLocalSleepChange(TObject *Sender)
{
   EditLocalSleep->Text = IntToStr(TrackBarLocalSleep->Position);
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::TrackBarAChange(TObject *Sender)
{
   EditA->Text = FloatToStr((float)TrackBarA->Position/1000);
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::TrackBarTemp0Change(TObject *Sender)
{
   EditTemp0->Text = IntToStr(TrackBarTemp0->Position);
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::TrackBarPercentAcChange(TObject *Sender)
{
   EditPercentAc->Text = IntToStr(TrackBarPercentAc->Position);
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::TrackBarTempPlusChange(TObject *Sender)
{
   EditTempPlus->Text = IntToStr(TrackBarTempPlus->Position);
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::TrackBarLkChange(TObject *Sender)
{
   EditLk->Text = IntToStr(TrackBarLk->Position);
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::TrackBarAlfaChange(TObject *Sender)
{
   EditAlfa->Text = FloatToStr((float)TrackBarAlfa->Position/100);
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::TrackBarStopChange(TObject *Sender)
{
   EditStop->Text = FloatToStr((float)TrackBarStop->Position/100);
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::EditMinXKeyPress(TObject *Sender, char &Key)
{
   if (Key==VK_RETURN)
   {
      float fData = StrToFloat(EditMinX->Text);
      if ((fData >= ScrollBarMinX->Min) && (fData <= ScrollBarMinX->Max))
      {
         CalcFunc3DValue(iFirstVar, iSecondVar);
         Redraw();
  //       ScrollBarMinX->Position = (int)fData;
      }
      else
      {
         ShowMessage("Wrong data !");
         return;
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::EditMinYKeyPress(TObject *Sender, char &Key)
{
   if (Key==VK_RETURN)
   {
      float fData = StrToFloat(EditMinY->Text);
      if ((fData >= ScrollBarMinY->Min) && (fData <= ScrollBarMinY->Max))
      {
         CalcFunc3DValue(iFirstVar, iSecondVar);
         Redraw();
  //       ScrollBarMinY->Position = (int)fData;
      }
      else
      {
         ShowMessage("Wrong data !");
         return;
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::EditMaxXKeyPress(TObject *Sender, char &Key)
{
   if (Key==VK_RETURN)
   {
      float fData = StrToFloat(EditMaxX->Text);
      if ((fData >= ScrollBarMaxX->Min) && (fData <= ScrollBarMaxX->Max))
      {
         CalcFunc3DValue(iFirstVar, iSecondVar);
         Redraw();
 //        ScrollBarMaxX->Position = (int)fData;
      }
      else
      {
         ShowMessage("Wrong data !");
         return;
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TFormMain::EditMaxYKeyPress(TObject *Sender, char &Key)
{
   if (Key==VK_RETURN)
   {
      float fData = StrToFloat(EditMaxY->Text);
      if ((fData >= ScrollBarMaxY->Min) && (fData <= ScrollBarMaxY->Max))
      {
         CalcFunc3DValue(iFirstVar, iSecondVar);
         Redraw();
     //    ScrollBarMaxY->Position = (int)fData;
      }
      else
      {
         ShowMessage("Wrong data !");
         return;
      }
   }
}
//---------------------------------------------------------------------------

